/*
 * var.h - Variable handling
 *
 * Written 2001,2002 by Werner Almesberger
 * Copyright 2001 EPFL-ICA, Network Robots
 * Copyright 2002 Network Robots
 */


#ifndef VAR_H
#define VAR_H

#include "data.h"
#include "location.h"


typedef enum {
    vu_new,		/* just created */
    vu_unused,		/* initialized but never read */
    vu_used,		/* also read */
    vu_forward		/* forward-declaration, will be set later */
} VAR_USAGE;

typedef struct _var {
    const char *name;
    DATA data;
    DATA *data_ref; /* for forward-declared variables */
    int level;
    VAR_USAGE usage;
    LOCATION location;
    LOCATION found; /* new location if redefining */
    struct _var *next;
} VAR;


VAR *var_find(const char *name);

void var_begin_scope(void);
void var_end_scope(void);

void var_outer_scope(void);

/*
 * var_outer_scope is like var_end_scope, except that it doesn't remove
 * variables.
 */

void var_begin_access(VAR *var);
void var_follow(const char *name);

void var_push_access(void);
void var_pop_access(void);

void var_set(DATA data);
DATA *var_forward(void); /* also allocates data */
int var_initialized(const VAR *var);
DATA var_get(void);

#endif /* VAR_H */
